

#include <avr/io.h>
#include <avr/interrupt.h>
#include "atx_spid_treiber.h"

//#define SPID_USEINT       // Interrupt ein

#define Spid_Busy()          while(!(SPID.STATUS & (1<<SPI_IF_bp)))


volatile uint8_t spidTransferComplete;

// SPI interrupt service handler
#ifdef SPID_USEINT
ISR(SPID_INT_vect)
{
	spidTransferComplete = true;
}
#endif


//********************
#include "tools.h"

//*----------------------------------------------------------------------------
// SPI-Bus ADT7301 SOT-23 Pin Configuration
//*----------------------------------------------------------------------------
//													
#define SPID_CS      4   // PD4 (Pin 24)  	->   	PIN 5 
#define SPID_MOSI    5   // PD5 (Pin 25) 	->		PIN 2
#define SPID_MISO    6   // PD6 (Pin 26) 	-> 		PIN 6
#define SPID_SCK     7   // PD7 (Pin 27) 	->		PIN 4




void spidPort_Init(void)
{
	// === PORT-B CONFIG ===
	PORTD_DIRSET  = (1 << SPID_MOSI) | (1 << SPID_SCK)  | (1 << SPID_CS);
	PORTD_DIRCLR  = (1 << SPID_MISO);    
	PORTD_OUTSET = 	(1 << SPID_CS);	   //SS/CS high
}


// access routines
void spidInit()
{
	spidPort_Init();           //max Spi-Takz 16 Mhz
	
	SPID.CTRL =  SPI_MODE_0_gc | SPI_PRESCALER_DIV4_gc  | (1<< SPI_CLK2X_bp) | (1<< SPI_ENABLE_bp) | (1<< SPI_MASTER_bp);
		
    SPID.STATUS = 0;

	spidTransferComplete = true;

	// enable SPI interrupt
	#ifdef SPID_USEINT
	SPID.INTCTRL = SPI_INTLVL_MED_gc;

	#endif
}

void spidSendByte(uint8_t data)
{
	// send a byte over SPI and ignore reply
	#ifdef SPID_USEINT
		while(!spidTransferComplete);
		spidTransferComplete = FALSE;
	#else
		Spid_Busy();
	#endif

	SPID_DATA=data;
}

uint8_t spidTransferByte(uint8_t data)
{
	#ifdef SPID_USEINT
	// send the given data
	spidTransferComplete = FALSE;
	SPID_DATA=data;
	// wait for transfer to complete
	while(!spidTransferComplete);
	#else
	// send the given data
	SPID_DATA=data;
	// wait for transfer to complete
	Spid_Busy();
	#endif
	// return the received data
	return data=SPID_DATA;
}

uint16_t spidTransferWord(uint16_t data)
{
	uint16_t rxData = 0;

	// send MS byte of given data
	rxData = (spidTransferByte((data>>8) & 0x00FF))<<8;
	// send LS byte of given data
	rxData |= (spidTransferByte(data & 0x00FF));

	// return the received data
	return rxData;
}
